#!/usr/bin/env python3
import logging
from .parsers import parse_args, parse_conf
from .helpers import validChrootName


def main():
    args = parse_args()
    config_data = parse_conf()
    logging.debug(f"Initialization complete.")

    # Validate chroot name
    try:
        chroot_name = vars(args)["chroot-name"]
        logging.debug("Validating chroot name")
        if not validChrootName(config_data, chroot_name):
            logging.debug("Chroot name is invalid, exitting")
            exit(2)
    except KeyError:
        # chroot name is not required
        pass

    # run program depending on what the arguments are
    logging.debug(f"Running with args {args}")
    args.func(config_data, vars(args))


if __name__ == "__main__":
    main()



from termcolor import colored
from .helpers import (
    chkMountStatus,
    findLocation,
    getChrootCommand,
    ensureMount,
    suRunCommand,
)
import logging


def help(config_data, args):
    print(f"{colored('run', 'yellow')} {colored('chrootman', 'green')} -h for help")
    logging.debug(f"Arg not specified: {config_data}, {args}")


def list(config_data, args):
    logging.debug(f"Arg is not used {args}")
    print(f"{colored('Available','green')} {colored('chroots', 'yellow')}:")
    for item in config_data["chroots"]:
        print(colored(item, "yellow"))


# Show info of specified chroot
def showinfo(config_data, args):
    chroot_name = args["chroot_name"]
    print(f"{colored('Name','green')}: \t\t{colored(chroot_name, 'yellow')}")
    try:
        print(
            f"{colored('Description', 'green')}: \t{colored(config_data['chroots'][chroot_name]['description'], 'yellow')}"
        )
    except:
        logging.warn("Description not found, skipping")
    print(
        f"{colored('Location', 'green')}: \t{colored(findLocation(config_data, chroot_name), 'yellow')}"
    )
    print(
        f"{colored('Distro', 'green')}: \t{colored(config_data['chroots'][chroot_name]['distro'], 'yellow')}"
    )


# Execute mount-command from settings
def mount(config_data, args, chroot_name=None):
    if not chroot_name:
        chroot_name = args["chroot_name"]
    if chkMountStatus(config_data, chroot_name):
        logging.error(f"{chroot_name} is already mounted.")
        exit(1)
    su_provider = config_data["general"]["su-provider"]
    distro = config_data["chroots"][chroot_name]["distro"]

    # check whether to use default or not
    mount_command = getChrootCommand(config_data, "mount-command", distro, chroot_name)
    suRunCommand(config_data, chroot_name, su_provider, mount_command, "mount_command")


def unmount(config_data, args):
    chroot_name = args["chroot_name"]
    su_provider = config_data["general"]["su-provider"]
    distro = config_data["chroots"][chroot_name]["distro"]

    if not chkMountStatus(config_data, chroot_name):
        logging.error(f"{chroot_name} is not mounted.")
        exit(1)

    unmount_command = getChrootCommand(
        config_data,
        "unmount-command",
        distro,
        chroot_name,
    )
    suRunCommand(
        config_data, chroot_name, su_provider, unmount_command, "unmount_command"
    )


def login(config_data, args):
    chroot_name = args["chroot_name"]

    if ensureMount(config_data, chroot_name, args, mount, login):
        return

    su_provider = config_data["general"]["su-provider"]
    distro = config_data["chroots"][chroot_name]["distro"]

    login_command = getChrootCommand(config_data, "login-command", distro, chroot_name)
    suRunCommand(config_data, chroot_name, su_provider, login_command, "login-command")


def launch(config_data, args):
    chroot_name = args["chroot_name"]

    if ensureMount(config_data, chroot_name, args, mount, launch):
        return

    su_provider = config_data["general"]["su-provider"]
    distro = config_data["chroots"][chroot_name]["distro"]
    launch_command = getChrootCommand(
        config_data, "launch-command", distro, chroot_name
    )
    suRunCommand(config_data, chroot_name, su_provider, launch_command, "login-command")

    unmount(config_data, args)


def updateChroot(config_data, args):
    chroot_name = args["chroot_name"]
    su_provider = config_data["general"]["su-provider"]
    distro = config_data["chroots"][chroot_name]["distro"]
    update_command = getChrootCommand(
        config_data, "update-command", distro, chroot_name
    )

    if ensureMount(config_data, chroot_name, args, mount, updateChroot):
        return

    suRunCommand(
        config_data, chroot_name, su_provider, update_command, "update_command"
    )


def update(config_data, args):
    if not args["chroot_name"]:
        logging.debug("chroot_name arg is not found")
        if not args["all"]:
            print(
                f"{colored('chroot_name', 'yellow')} not specified, assuming {colored('--all', 'green')}"
            )

        for chroot in config_data["chroots"]:
            print(f" {colored('Updating', 'green')}: {colored(chroot, 'yellow')}")
            args["chroot_name"] = chroot
            updateChroot(config_data, args)

    else:
        logging.debug(f"Updating {args['chroot_name']}")
        updateChroot(config_data, args)



import subprocess
from os import environ
from logging import debug, error
from termcolor import colored


def cliAskChoice():
    yes = {"yes", "yep", "y", "ye", ""}
    no = {"no", "nope", "n"}

    choice = input(f"{colored('Input', 'blue')}: ({colored('y','green')}/{colored('n', 'red')}) > ").lower()
    debug("input is:" + choice)
    if choice in yes:
        debug("Choice is yes")
        return True
    elif choice in no:
        debug("Choice is no")
        return False
    else:
        print(f" Please respond with '{colored('y', 'green')}' or '{colored('n', 'red')}'")
        return cliAskChoice()


def findLocation(config_data, chroot_name):
    if not validChrootName(config_data, chroot_name):
        exit(1)

    if "custom-location" not in config_data["chroots"][chroot_name]:
        return (
            config_data["general"]["unified-location"]
            + config_data["chroots"][chroot_name]["location"]
        )
    else:
        return config_data["chroots"][chroot_name]["custom-location"]


def validChrootName(config_data, chroot_name):
    if chroot_name not in config_data["chroots"]:
        error(f"{chroot_name} is not in the configuration file!")
        error(f"Available options are: {list(config_data['chroots'])}")
        return False
    else:
        return True


def getChrootCommand(config_data, command_name, distro = None, chroot_name = None):
    # chroots-settings -> distro-settings -> default-distro-settings
    debug(f"distro: {distro}, chroot_name: {chroot_name}, command_name: {command_name}")
    try:
        command = config_data["chroots"][chroot_name][command_name]
    except KeyError:
        try:
            command = config_data["general"]["distro-settings"][distro][command_name]
        except KeyError:
            try:
                debug("distro is not in configured list, using default")
                command = config_data["general"]["distro-settings"]["default"][command_name]
            except KeyError:
                error("KeyError: Make sure you have properly configured the chroots")
                exit(1)

    command = command.replace(
        "$rootfs_location", findLocation(config_data, chroot_name)
    )
    command = command.replace("~", environ["HOME"])
    debug(f"Command loaded:\n{command}")
    return command


def ensureMount(config_data, chroot_name, args, mount_func, command_func):
    if not chkMountStatus(config_data, chroot_name):
        error("Filesystem is not mounted! Do you want to mount it first?")
        if cliAskChoice():
            mount_func(config_data, args)
            command_func(config_data, args)
            return True
    return False


def chkMountStatus(config_data, chroot_name):
    chrootPath = findLocation(config_data, chroot_name).replace("~", environ["HOME"])
    output = subprocess.run(["mount"], capture_output=True).stdout
    if chrootPath in str(output):
        return True
    else:
        return False


def suRunCommand(config_data, chroot_name, su_provider, command, command_type):
    if validChrootName(config_data, chroot_name):
        print(f"→ {colored('Executing', 'green')} {colored(command_type, 'yellow')}")
        child = subprocess.run([su_provider, "sh", "-c", command])
        print(f"⏎ {colored(command_type, 'yellow')} executed and returned {colored(str(child.returncode), 'magenta')}")



import argparse
import os

# logging
import logging
import yaml

# functionality
from .actions import *
from os import environ
from .helpers import cliAskChoice
from shutil import copy


class custom_formatter(logging.Formatter):
    cyan = "\x1b[36m"
    blue = "\x1b[34m"
    yellow = "\x1b[33m"
    red = "\x1b[31m"
    reset = "\x1b[0m"

    def __init__(self, fmt):
        super().__init__()
        self.fmt = fmt
        self.FORMATS = {
            logging.DEBUG: self.cyan + self.fmt + self.reset,
            logging.INFO: self.blue + self.fmt + self.reset,
            logging.WARNING: self.yellow + self.fmt + self.reset,
            logging.ERROR: self.red + self.fmt + self.reset,
            logging.CRITICAL: self.red + self.fmt + self.reset,
        }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


def add_subcommands(subparsers):
    # add parsers for subcommands
    list_parser = subparsers.add_parser("list", help="Display availble chroots")
    list_parser.set_defaults(func=list)

    info_parser = subparsers.add_parser(
        "info", help="Display information on specified chroot"
    )
    info_parser.add_argument("chroot_name", type=str, help="specify the name of chroot")
    info_parser.set_defaults(func=showinfo)

    mount_parser = subparsers.add_parser(
        "mount", help="Mount filesystem of specified chroot"
    )
    mount_parser.add_argument(
        "chroot_name", type=str, help="specify name of chroot to mount"
    )
    mount_parser.set_defaults(func=mount)

    unmount_parser = subparsers.add_parser(
        "unmount", help="unmount filesystem of specified chroot"
    )
    unmount_parser.add_argument(
        "chroot_name", type=str, help="specify name of chroot to unmount"
    )
    unmount_parser.set_defaults(func=unmount)

    login_parser = subparsers.add_parser("login", help="login to specified chroot")
    login_parser.add_argument(
        "chroot_name", type=str, help="specify name of chroot to login"
    )
    login_parser.set_defaults(func=login)

    launch_parser = subparsers.add_parser(
        "launch", help="launch chroot using command specified in chroot"
    )
    launch_parser.add_argument(
        "chroot_name", type=str, help="specify name of chroot to launch"
    )
    launch_parser.set_defaults(func=launch)

    update_parser = subparsers.add_parser("update", help="update to specified chroot")
    update_parser.add_argument(
        "chroot_name",
        nargs="?",
        type=str,
        help="specify name of chroot to update",
    )
    update_parser.add_argument("--all", "-a", action="store_true")
    update_parser.set_defaults(func=update)


# read args and set up debugger based on flags
def parse_args():
    parser = argparse.ArgumentParser(description="Manage chroot instances")
    parser.set_defaults(func=help)

    parser.add_argument(
        "-d",
        "--debug",
        help="Increase output verbosity to display dibugging messages",
        action="store_true",
    )

    subparsers = parser.add_subparsers(title="available subcommands")

    add_subcommands(subparsers)
    # argument
    args = parser.parse_args()

    # set up debugger
    logger = logging.getLogger()
    if args.debug:
        logger.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.ERROR)

    log = logging.StreamHandler()
    fmt = "%(levelname)s: %(message)s"
    log.setFormatter(custom_formatter(fmt))

    logger.addHandler(log)

    logging.debug("Successfully parsed arguments")

    return args


def parse_conf():

    # Set config location based on Env vars
    config_location = environ.get("XDG_CONFIG_HOME")
    if not config_location:
        config_location = "~/.chrootman.yaml"
    else:
        config_location = config_location + "/chrootman/config.yaml"

    logging.debug(f"config_location is set to {config_location}")

    # Open file, otherwise return None
    try:
        f = open(config_location, mode="r")
        logging.debug("Reading file")
        config_data = yaml.load(f, Loader=yaml.CLoader)
        f.close()
        return config_data
    except:
        logging.warning("Config file not found!")
        logging.error(
            "Config file not found! Do you want to install a default configuration file?"
        )
        if cliAskChoice() == True:
            # path of the directory of script
            if not __file__:
                path = __path__
            else:
                path = os.path.dirname(os.path.abspath(__file__))

            copy(f"{path}/files/config.yaml", config_location)

            logging.debug("Installed default config file.")
            return parse_conf()
        else:
            logging.error("A configuration file is required.")
            logging.debug("Exitting...")
            exit()
